package cn.com.qmth.examcloud.core.basic.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import javax.persistence.criteria.Join;
import javax.persistence.criteria.Predicate;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.ByteUtil;
import cn.com.qmth.examcloud.commons.base.util.SHA256;
import cn.com.qmth.examcloud.commons.base.util.StringUtil;
import cn.com.qmth.examcloud.commons.web.security.AccessUserOps;
import cn.com.qmth.examcloud.commons.web.security.AccessUserOpsForRedis;
import cn.com.qmth.examcloud.commons.web.security.entity.AccessUser;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.core.basic.base.enums.AccountType;
import cn.com.qmth.examcloud.core.basic.base.enums.UserType;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.ThirdPartyAccessRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserLoginRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserOpsLogRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.Org;
import cn.com.qmth.examcloud.core.basic.dao.entity.ThirdPartyAccessEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.ThirdPartyAccessPK;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserLogin;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserOpsLog;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRole;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.UserService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;
import cn.com.qmth.examcloud.core.basic.service.bean.UserInfo;

/**
 * 用户服务类 Created by songyue on 17/1/13.
 */
@Service
public class UserServiceImpl implements UserService {

	@Autowired
	UserRepo userRepo;

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	OrgService orgService;

	@Autowired
	UserRoleRepo userRoleRepo;

	@Autowired
	UserLoginRepo userLoginRepo;

	@Autowired
	UserOpsLogRepo userOpsLogRepo;

	@Autowired
	ThirdPartyAccessRepo thirdPartyAccessDao;

	RedisTemplate redisTemplate;

	@Autowired
	AuthService authService;

	AccessUserOps accessUserOps;

	@Autowired
	public UserServiceImpl(RedisTemplate redisTemplate) {
		this.redisTemplate = redisTemplate;
		this.accessUserOps = new AccessUserOpsForRedis(redisTemplate);
	}

	/**
	 * 一般登录
	 * 
	 * @param loginName
	 * @param password
	 * @return
	 */
	public UserInfo login(String loginName, String password) throws Exception {
		UserEntity user = userRepo.findByLoginName(loginName);
		if (user == null) {
			throw new RuntimeException("该用户不存在");
		} else if (!user.getEnable()) {
			throw new RuntimeException("该用户被禁用");
		}
		return loginProcess(user);

	}

	/**
	 * 二级登录
	 * 
	 * @param orgId
	 * @param loginName
	 * @param password
	 * @return
	 */
	public UserInfo login(long orgId, String loginName, String password) throws Exception {
		UserEntity user = userRepo.findByRootOrgIdAndLoginName(orgId, loginName);
		if (user == null) {
			throw new RuntimeException("该用户不存在");
		} else if (!user.getEnable()) {
			throw new RuntimeException("该用户被禁用");
		}
		return loginProcess(user);
	}

	/**
	 * 登录处理 重新定义 by wangwei
	 * 
	 * @param user
	 * @return
	 */
	public UserInfo loginProcess(UserEntity user) throws Exception {
		LoginInfo loginInfo = new LoginInfo();
		loginInfo.setAccountType(AccountType.COMMON_LOGIN_NAME.getCode());
		loginInfo.setAccountValue(user.getLoginName());
		loginInfo.setPassword(user.getPassword());
		loginInfo.setRootOrgId(String.valueOf(user.getRootOrgId()));
		cn.com.qmth.examcloud.commons.web.security.bean.User loginUser = authService
				.login(loginInfo);
		String userToken = loginUser.getUserToken();
		return getUserInfo(user, userToken);
	}

	/**
	 * 第三方接入
	 *
	 * @author WANGWEI
	 * @param orgId
	 * @param userid
	 * @param appid
	 * @param timestamp
	 * @param token
	 * @return
	 * @throws Exception
	 */
	@Deprecated
	public UserInfo thirdPartyAccess(long orgId, String userid, String appid, String timestamp,
			String token) throws Exception {
		ThirdPartyAccessEntity thirdPartyAccess = thirdPartyAccessDao
				.findOne(new ThirdPartyAccessPK(orgId, appid));

		if (null == thirdPartyAccess) {
			throw new RuntimeException("第三方系统接入信息未配置!");
		}

		long timestampLong = 0L;
		try {
			timestampLong = Long.parseLong(timestamp);
		} catch (Exception e) {
			throw new RuntimeException("timestamp错误");
		}

		if (Math.abs(System.currentTimeMillis() - timestampLong) > thirdPartyAccess
				.getTimeRange()) {
			throw new RuntimeException("第三方系统接入鉴权失败: timestamp超出时间差范围!");
		}

		String secretKey = thirdPartyAccess.getSecretKey();
		String joinStr = StringUtil.join(userid, orgId, appid, timestamp, secretKey);
		byte[] bytes = SHA256.encode(joinStr);
		String hexAscii = ByteUtil.toHexAscii(bytes);

		if (!hexAscii.equals(token)) {
			throw new RuntimeException("第三方系统接入鉴权失败: token校验失败!");
		}

		UserEntity user = userRepo.findByRootOrgIdAndLoginName(orgId, userid);
		if (user == null) {
			throw new RuntimeException("该用户不存在!");
		}

		return loginProcess(user);
	}

	/**
	 * 初始化用户登录
	 * 
	 * @param user
	 */
	@Deprecated
	public void initUserLogin(UserEntity user) {
		// 判断是否已登录,若已登录则强制已登录用户退出
		UserLogin userLogin = userLoginRepo.findFirstByUserId(user.getId());
		if (userLogin != null) {
			accessUserOps.delete(userLogin.getToken());
			userLoginRepo.deleteByUserId(userLogin.getUserId());
		}
	}

	/**
	 * 创建用户操作日志
	 * 
	 * @param user
	 * @param action
	 */
	public void createUserOpsLog(UserEntity user, String action) {
		UserOpsLog userOpsLog = new UserOpsLog(action, user.getId(), user.getName(), new Date());
		userOpsLogRepo.save(userOpsLog);
	}

	/**
	 * 创建用户登录记录
	 * 
	 * @param token
	 * @param user
	 */
	@Deprecated
	public void createUserLogin(String token, UserEntity user) {
		UserLogin userLogin = new UserLogin();
		userLogin.setLoginTime(new Date());
		userLogin.setUserId(user.getId());
		userLogin.setToken(token);
		userLoginRepo.save(userLogin);
	}

	/**
	 * 获取用户信息DTO
	 * 
	 * @param user
	 * @param token
	 * @return
	 */
	public UserInfo getUserInfo(UserEntity user, String token) throws Exception {
		UserInfo userInfo = new UserInfo();
		Org org = orgService.findOne(user.getOrgId());
		Org rootOrg = orgService.findOne(user.getRootOrgId());
		userInfo.setUserId(user.getId());
		userInfo.setOrgId(user.getOrgId());
		userInfo.setRootOrgId(user.getRootOrgId());
		userInfo.setName(user.getName());
		userInfo.setAvatar(user.getAvatar());
		userInfo.setLoginName(user.getLoginName());
		if (org != null) {
			userInfo.setOrgName(org.getName());
		}
		if (rootOrg != null) {
			userInfo.setRootOrgLogo(rootOrg.getLogo());
			userInfo.setRootOrgName(rootOrg.getName());
		}
		userInfo.setToken(token);
		return userInfo;
	}

	/**
	 * 获取角色名称
	 * 
	 * @param userRoles
	 * @return
	 */
	public List<String> getRoleNames(List<UserRole> userRoles) {

		List<String> roleNameList = userRoles.stream()
				.map(userRole -> RoleMeta.valueOf(userRole.getRoleCode()).getName())
				.collect(Collectors.toList());
		return roleNameList;

	}

	/**
	 * 登出
	 * 
	 * @param accessUser
	 */
	public void logout(AccessUser accessUser) {
		accessUserOps.delete(accessUser.getToken());
		userLoginRepo.deleteByUserId(accessUser.getUserId());
	}

	public List<UserEntity> getMarker(Long rootOrgId) {
		return userRepo.findMarkerByRootOrgId(rootOrgId);
	}

	/**
	 * 查询评卷员带分页
	 * 
	 * @param user
	 * @param pageable
	 * @return
	 */
	public Page<UserEntity> getAllMaker(UserEntity user, Pageable pageable) {
		Specification<UserEntity> specification = getSpecification(user);
		return userRepo.findAll(specification, pageable);
	}

	public Specification<UserEntity> getSpecification(UserEntity user) {
		Specification<UserEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("type"), UserType.COMMON));
			predicates.add(cb.equal(root.get("rootOrgId"), user.getRootOrgId()));
			Join join = root.join("userRoles");
			predicates.add(cb.equal(join.get("roleCode"), RoleMeta.MARKER.name()));
			if (StringUtils.isNotEmpty(user.getLoginName())) {
				predicates.add(cb.like(root.get("loginName"), "%" + user.getLoginName() + "%"));
			}
			if (StringUtils.isNotEmpty(user.getName())) {
				predicates.add(cb.like(root.get("name"), "%" + user.getName() + "%"));
			}
			if (user.getEnable() != null) {
				predicates.add(cb.equal(root.get("enable"), user.getEnable()));
			}
			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		return specification;
	}

	public UserEntity save(UserEntity user) {
		UserEntity one = userRepo.findByRootOrgIdAndLoginName(user.getRootOrgId(),
				user.getLoginName());
		if (one != null && (!one.getId().equals(user.getId()))) {
			throw new StatusException("B-150002", "登录名已存在");
		}
		return userRepo.save(user);
	}

	@Override
	public UserEntity getUser(Long rootOrgId, String loginName) {
		UserEntity user = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);
		return user;
	}

}
